/**
 * Copyright (c) 2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock;

import java.net.InetAddress;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.management.ObjectName;

import org.ow2.petals.jmx.api.api.AdminServiceClient;
import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.DeploymentServiceClient;
import org.ow2.petals.jmx.api.api.EndpointRegistryClient;
import org.ow2.petals.jmx.api.api.InstallationServiceClient;
import org.ow2.petals.jmx.api.api.InstallerComponentClient;
import org.ow2.petals.jmx.api.api.JMXClient;
import org.ow2.petals.jmx.api.api.LoggerServiceClient;
import org.ow2.petals.jmx.api.api.PetalsAdminServiceClient;
import org.ow2.petals.jmx.api.api.RouterMonitorServiceClient;
import org.ow2.petals.jmx.api.api.RuntimeConfigurationComponentClient;
import org.ow2.petals.jmx.api.api.SystemMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.TopologyServiceClient;
import org.ow2.petals.jmx.api.api.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.ConnectionErrorException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.DeploymentServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.EndpointRegistryServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.EndpointRegistryServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallerComponentDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.LoggerServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.PetalsAdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.RouterMonitorServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.RouterMonitorServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.RuntimeConfigurationDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.RuntimeConfigurationErrorException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.SystemMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.exception.TopologyServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.DefectFilter;
import org.ow2.petals.jmx.api.api.monitoring.DefectListener;
import org.ow2.petals.jmx.api.api.monitoring.LocalTransporterMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.ComponentMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.soap.PetalsBcSoapMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.exception.DefectEmitterDoesNotExistsException;
import org.ow2.petals.jmx.api.api.monitoring.exception.DefectListenerException;
import org.ow2.petals.jmx.api.api.monitoring.exception.LocalTransporterMonitoringServiceDoesNotExistException;
import org.ow2.petals.jmx.api.api.monitoring.exception.LocalTransporterMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.exception.NoDefectListenerException;
import org.ow2.petals.jmx.api.mock.monitoring.DefaultDefectCreator;
import org.ow2.petals.jmx.api.mock.monitoring.DefaultDefectEmitterMock;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;
import org.ow2.petals.jmx.api.mock.monitoring.component.soap.PetalsBcSoapMonitoringServiceClientMock;

/**
 * Mock of {@link JMXClient} for unit tests of application using the Petals JMX
 * API
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class JMXClientMock implements JMXClient {

    /**
     * The defect emitters for which a subscription was subscribed.
     */
    private Map<String, Thread> defectEmitters = new ConcurrentHashMap<String, Thread>();

    public JMXClientMock(final String host, final Integer jndiPort, final String username,
            final String password) {
        // TODO Auto-generated constructor stub
    }

    @Override
    public InetAddress getHost() throws NotRemoteJMXClientException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public int getPort() throws NotRemoteJMXClientException {
        // TODO Auto-generated method stub
        return 0;
    }

    @Override
    public String getUsername() throws NotRemoteJMXClientException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public PetalsAdminServiceClient getPetalsAdminServiceClient()
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException,
            ConnectionErrorException {
        return new PetalsAdminServiceClientMock();
    }

    @Override
    public AdminServiceClient getAdminServiceClient() throws AdminDoesNotExistException,
            AdminServiceErrorException, ConnectionErrorException {
        return new AdminServiceClientMock();
    }

    @Override
    public InstallationServiceClient getInstallationServiceClient()
            throws InstallationServiceDoesNotExistException, InstallationServiceErrorException,
            ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public DeploymentServiceClient getDeploymentServiceClient()
            throws DeploymentServiceDoesNotExistException, DeploymentServiceErrorException,
            ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public ComponentClient getComponentClient(String componentName)
            throws ComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public ComponentClient getComponentClient(ObjectName componentMBeanName)
            throws ComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public InstallerComponentClient getInstallerComponentClient(String componentName)
            throws InstallerComponentDoesNotExistException, ComponentErrorException,
            ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public EndpointRegistryClient getEndpointRegistryClient() throws ConnectionErrorException,
            EndpointRegistryServiceDoesNotExistException, EndpointRegistryServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public SystemMonitoringServiceClient getSystemMonitoringServiceClient()
            throws ConnectionErrorException, SystemMonitoringDoesNotExistException,
            SystemMonitoringServiceErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public RuntimeConfigurationComponentClient getRuntimeConfigurationComponentClient(
            String dynamicComponentName) throws RuntimeConfigurationDoesNotExistException,
            RuntimeConfigurationErrorException, ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public RouterMonitorServiceClient getRouterMonitoringServiceClient()
            throws RouterMonitorServiceDoesNotExistException, RouterMonitorServiceErrorException,
            ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public LoggerServiceClient getLoggerServiceClient() throws LoggerServiceErrorException,
            LoggerServiceDoesNotExistException, ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public LocalTransporterMonitoringServiceClient getLocalTransporterMonitoringServiceClient()
            throws LocalTransporterMonitoringServiceErrorException,
            LocalTransporterMonitoringServiceDoesNotExistException, ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock processing:
     * <ul>
     * <li>
     * If
     * {@link ComponentMonitoringServiceClientMock#IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT}
     * is used as <code>componentIdentifier</code>, the exception
     * {@link ComponentMonitoringServiceDoesNotExistException} is thrown.</li>
     * </ul>
     * </p>
     */
    @Override
    public ComponentMonitoringServiceClient getComponentMonitoringServiceClient(
            String componentIdentifier) throws ComponentMonitoringServiceErrorException,
            ComponentMonitoringServiceDoesNotExistException, ConnectionErrorException {
        if (ComponentMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT
                .equals(componentIdentifier)) {
            throw new ComponentMonitoringServiceDoesNotExistException();
        } else {
            return new ComponentMonitoringServiceClientMock(componentIdentifier);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock processing:
     * <ul>
     * <li>
     * If
     * {@link PetalsBcSoapMonitoringServiceClientMock#IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT}
     * is used as <code>componentIdentifier</code>, the exception
     * {@link ComponentMonitoringServiceDoesNotExistException} is thrown.</li>
     * </ul>
     * </p>
     */
    @Override
    public PetalsBcSoapMonitoringServiceClient getPetalsBcSoapMonitoringServiceClient(
            String componentIdentifier) throws ComponentMonitoringServiceErrorException,
            ComponentMonitoringServiceDoesNotExistException, ConnectionErrorException {
        if (PetalsBcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT
                .equals(componentIdentifier)) {
            throw new ComponentMonitoringServiceDoesNotExistException();
        } else {
            return new PetalsBcSoapMonitoringServiceClientMock(componentIdentifier);
        }
    }

    @Override
    public void subscribeToComponentMonitoringService(final String componentIdentifier,
            final DefectListener defectListener, final DefectFilter defectFilter)
            throws DefectListenerException, DefectEmitterDoesNotExistsException {
        if (PetalsBcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT
                .equals(componentIdentifier)) {
            throw new DefectEmitterDoesNotExistsException(componentIdentifier);
        } else if (PetalsBcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR
                .equals(componentIdentifier)) {
            throw new DefectListenerException(new Exception("Error on subscription"));
        } else {
            if (this.defectEmitters.containsKey(componentIdentifier)) {
                throw new DefectListenerException(
                        "A subscription already exists for this identifier '" + componentIdentifier
                                + "'");
            } else {
                final Thread defectEmitter = new DefaultDefectEmitterMock(
                        new DefaultDefectCreator(), defectListener);
                this.defectEmitters.put(componentIdentifier, defectEmitter);
                defectEmitter.start();
            }
        }
    }

    @Override
    public void unsubscribeToComponentMonitoringService(final String componentIdentifier)
            throws NoDefectListenerException, DefectEmitterDoesNotExistsException,
            DefectListenerException {
        if (PetalsBcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_AN_UNEXISTING_COMPONENT
                .equals(componentIdentifier)) {
            throw new DefectEmitterDoesNotExistsException(componentIdentifier);
        } else if (PetalsBcSoapMonitoringServiceClientMock.IDENTIFIER_FOR_COMPONENT_WITH_ERROR
                .equals(componentIdentifier)) {
            throw new DefectListenerException(new Exception("Error on unsubscription"));
        } else {
            if (!this.defectEmitters.containsKey(componentIdentifier)) {
                throw new DefectListenerException("No subscription exists for this identifier '"
                        + componentIdentifier + "'");
            } else {
                final Thread defectEmitter = this.defectEmitters.remove(componentIdentifier);
                defectEmitter.interrupt();
            }
        }
    }

    @Override
    public TopologyServiceClient getTopologyServiceClient() throws TopologyServiceErrorException,
            TopologyServiceDoesNotExistException, ConnectionErrorException {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public void disconnect() throws ConnectionErrorException {
        // TODO Auto-generated method stub

    }

}
